" Author: Adrian Zalewski <aazalewski@hotmail.com>
" Description: Ember-template-lint for checking Handlebars files

scriptencoding utf-8

call ale#handlers#embertemplatelint#DefineLinter('handlebars')
